# Full House -- A crazy queue system
# Copyright (C) 2007, 2008  Jaime Huerta-Cepas, Jordi Burguet-Castell

# This file is part of Full House.
#
# Full House is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Full House is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Full House.  If not, see <http://www.gnu.org/licenses/>.

# BARK is Basic Action Requested Kindly, a protocol to send strings over the
# network. It is also the name of a module that implements this protocol.

# Every message is first compressed and then passed prefixed by its
# size as an string finished with 'X'.
# For example: "what are you waiting for!?", which has 26 characters,
# would be passed as "26Xwhat are you waiting for!?"
# (actually, with "what are you waiting for" compressed)

import socket,sys
from zlib import compress, decompress

def send(sock, msg, print_status=False):
    """Send a message using the BARK protocol."""

    # Put the message preceded by its size in chars, and an 'X'
    msg = compress(msg)
    message = str(len(msg)) + 'X' + msg

    # Try to send it
    total_sent = 0
    while total_sent < len(message):
        try:
            sent = sock.send(message[total_sent:total_sent+1024])
        except:
            import traceback
            cla, exc, trbk = sys.exc_info()
            excName = cla.__name__
            try:
                excArgs = exc.__dict__["args"]
            except KeyError:
                excArgs = "<no args>"
            excTb = traceback.format_tb(trbk, 5)
            print >>sys.stderr, excName, excArgs, excTb
            raise

        if print_status:
            sys.stdout.write("\r%0.2f%% of %d" % ( (float(total_sent)/len(message))*100,len(message)) )
            sys.stdout.flush()
        
        if sent == 0:
            raise RuntimeError, 'Socket connection broken.'
        total_sent += sent


def sendall(sock, msg):
    """Send a message using the BARK protocol."""

    # Put the message preceded by its size in chars, and an 'X'
    msg = compress(msg)
    message = str(len(msg)) + 'X' + msg
    # Try to send it
    try:
        sock.sendall(message)
    except:
        import sys,traceback
        cla, exc, trbk = sys.exc_info()
        excName = cla.__name__
        try:
            excArgs = exc.__dict__["args"]
        except KeyError:
            excArgs = "<no args>"
        excTb = traceback.format_tb(trbk, 5)
        print >>sys.stderr, excName, excArgs, excTb
        raise

def recv(sock):
    """Receive a message using the BARK protocol."""

    # Size (as a string)
    ssize = ''
    while True:
        chunk = sock.recv(1)
        if   chunk == '':
            raise RuntimeError, 'Socket connection broken.'
        elif chunk == 'X':
            break
        ssize += chunk
    
    size = int(ssize)
    
    # Get the proper message
    message = ''
    while len(message) < size:
        buffer_size = size-len(message)
        if buffer_size >1024:
            buffer_size = 1024
        chunk = sock.recv( buffer_size )
        if chunk == '':  raise RuntimeError, 'Socket connection broken.'
        message += chunk
    
    return decompress(message)


  
def recv_old(sock):
    """Receive a message using the BARK protocol."""

    # Size (as a string)
    ssize = ''
    while True:
        chunk = sock.recv(1)
        if   chunk == '':
            raise RuntimeError, 'Socket connection broken.'
        elif chunk == 'X':
            break
        ssize += chunk
    
    size = int(ssize)
    
    # Get the proper message
    message = ''
    while len(message) < size:
        chunk = sock.recv(size-len(message))
        if chunk == '':  raise RuntimeError, 'Socket connection broken.'
        message += chunk
    
    return decompress(message)


def dic_send(sock, dic):
    """Send a dictionary using the BARK protocol."""

    send(sock, str(len(dic)))  # number of elements
    for k in dic:
        send(sock, k)
        send(sock, str(dic[k]))

    
def dic_recv(sock):
    """Receive a dictionary using the BARK protocol."""

    dic = {}
    n = int(recv(sock))  # number of elements

    for i in xrange(n):
        key   = recv(sock)
        value = recv(sock)
        dic[key] = value

    return dic
