# Full House -- A crazy queue system
# Copyright (C) 2007, 2008  Jaime Huerta-Cepas, Jordi Burguet-Castell

# This file is part of Full House.
#
# Full House is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Full House is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Full House.  If not, see <http://www.gnu.org/licenses/>.

# Common communication functions that will be used by the interfaces
# to ximo.

import os, cPickle, socket

import bark, common

HOST  = 'localhost'
PORT_OUTSIDE  = 31415
    

def ask_status():
    """Return the current scheduling status."""

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect((HOST, PORT_OUTSIDE))
    s.send('T')  # list
    status = bark.recv(s)
    s.close()

    return status


def get_skeletons():
    """Return a list with the projects skeletons."""

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect((HOST, PORT_OUTSIDE))
    s.send('L')  # list
    skeletons = cPickle.loads(bark.recv(s))
    s.close()

    return skeletons


def get_project(pname):
    """Return project 'pname' as it is now in ximo."""

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect((HOST, PORT_OUTSIDE))
    s.send('D')  # describe
    bark.send(s, pname)
    project = cPickle.loads(bark.recv(s))
    s.close()

    return project


def send_project(project):
    """Send project to ximo."""
    
    if not project.nodes:
        frontend, nodes, maxjobs, maxload = common.read_config()
        project.nodes = nodes
   
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect((HOST, PORT_OUTSIDE))
    try:
        s.send('A')  # submit a new project (add)
        bark.send(s, cPickle.dumps(project))
    except:
        pass
    
    s.close()


def kill_project(pname):
    """Send pname to kill."""

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect((HOST, PORT_OUTSIDE))
    s.send('K')  # submit a new project (add)
    bark.send(s, cPickle.dumps(pname))
    s.close()


def restart_jobs(pname,jobs):
    """Remarks as Waitting all given jobs."""
    
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect((HOST, PORT_OUTSIDE))
    s.send('M')  # restart
    bark.send(s, cPickle.dumps( (pname, jobs, 'W') ))
    s.close()
    

def set_properties(project):
    """Change project properties."""
    
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect((HOST, PORT_OUTSIDE))
    s.send('C')  # submit a new project (add)
    bark.send(s, cPickle.dumps(project))
    s.close()



def set_verbosity(level):
    """Set the verbosity of ximo to level 'level'."""

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect((HOST, PORT_OUTSIDE))
    s.send('V')  # set a new verbosity level
    bark.send(s, str(level))
    s.close()


def set_priority(pname, priority):
    """Change the priority of a project."""

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect((HOST, PORT_OUTSIDE))
    s.send('P')  # change project priority
    bark.send(s, pname)
    bark.send(s, str(priority))
    s.close()


def stop_scheduling():
    """Stop scheduling in ximo."""

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect((HOST, PORT_OUTSIDE))
    s.send('S')  # stop scheduling
    s.close()


def resume_scheduling():
    """Resume scheduling in ximo."""

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect((HOST, PORT_OUTSIDE))
    s.send('R')  # resume scheduling
    s.close()

    

def kill_ximo():
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect((HOST, PORT_OUTSIDE))
    s.send('Q')
    s.close()



# -----> UTILITY FUNCTIONS <-----

def get_usernames():
    users = {}
    for line in open('/etc/passwd'):
        parts = line.split(':')
        name, uid = parts[0], int(parts[2])

        users[uid] = name

    return users


def get_priorities(uid):
    if uid >= 600:  # external users
        return 1, 10

    return 1, 20
